pxm.c: Routines for reading and writing image files

The three formats supported are:

pbm: "portable bitmap" --- for binary data (black and white)
pgm: "portable graymap" --- for 8-bit grayscale (no color map)
ppm: "portable pixelmap" --- for 24-bit color (3 planes of 8-bit data)

The read routine is intelligent, in that it determines which type of file
is being read, and acts accordingly.  To accomodate this flexibility, I
have defined a data structure which can handle image data from any of the
three formats.  This definition can be found in pxm.h, and is repeated here
for convenience:

  struct pxm_img {
    int			height, width, max_gray;
    char		pxm_type; /* 'p', 'b', or 'g' */
    unsigned char	**mono; /* 'g' or 'b': mono[row][col] */
    unsigned char	***color; /* 'p': color[plane][row][col] */
  };

The function prototype for the read routine is:

  int read_pxm (FILE *fp, struct pxm_img *img);

You must open the file yourself, and provide the corresponding file pointer.
read_pxm does the rest including memory allocation.  If the image is binary 
or grayscale, it is stored in the 2-D array mono.  Otherwise, it is put in 
the 3-D array color.  You can tell which sort of image was read by checking 
pxm_type, which is set automatically.  The height and width are also provided 
for you.  Unless the image is binary, the max_gray parameter is also set, 
corresponding to the maximum gray level for a pgm file, and the maximum color 
component for a ppm file.  If the image is binary, it is still represented 
in memory as 8-bit data, where black is 0 and white is 255.  The function 
returns 0 if the read was successful; otherwise it returns 1.

The function prototype for the write routine is:

  int write_pxm (FILE *fp, struct pxm_img *img);

Again, you must open the file yourself.  You must pass a pointer to a valid 
pxm_img structure: height and width must be positive, and pxm_type must be
'p', 'b', or 'g'.  The max_gray value is computed by write_pxm, and overrides
any value you provide.  You must have the image data in the appropriate
array (mono for 'g' or 'b' file type, color for 'p').  Writing to 'p' or 
'g' files is straightforward enough; writing to 'b' involves first representing
the image in binary form.  This is done by write_pxm.  In other words, 
write_pxm converts the 8-bit array mono to a compact binary representation
and writes that to the file.  In doing so, it treats the 8-bit value 0 as
a 0, and any other 8-bit value as a 1.  The routine returns 0 if the write
was successful; otherwise it returns 1.

There is also a free routine:

  void free_pxm (struct pxm_img *img);

which is used to free an image array created by read_pxm.


I have tested these routines fairly carefully, but I don't guarantee them
to be bug-free.  If you have any problems, let me know (e-mail syd).


