#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "pxm.h"
#include "smap_io.h"
#include "alloc_util.h"
#include "allocate.h"



/* writes out a 2D file of class image data */ 
void write_img(
  IMAGE_CLASS **image, 	/* image with dimentions image[ht][wd] */
  int wd,               /* image width */
  int ht,               /* image height */
  char *name 		/* file name */
)
{
  FILE   *fp;
  struct pxm_img img;
  int i,j;

  fp = fopen(name,"wb");
  if (fp == NULL) {
    fprintf(stderr,"write_img: can't open file %s\n",name);
    exit (-1);
  }

  img.pxm_type = 'g';
  img.width = wd;
  img.height = ht;
  img.max_gray = 255;

  /* Convert segmentation image to unsigned char */
  img.mono = (unsigned char **)
                   get_img(img.width,img.height,sizeof(unsigned char));
  for(i=0; i<img.height; i++)
  for(j=0; j<img.width; j++) {
    img.mono[i][j] = (unsigned char)image[i][j];
  }

  if( write_pxm(fp, &img) ) {
    fprintf(stderr,"\nwrite_img: error writing out file %s\n",name);
    exit (-1);
  }

  free_img((void **)img.mono);

  fclose(fp);
}


/* reads a file 2D of character data */
void read_img(
  unsigned char	**image,  /* image with dimentions image[ht][wd] */
  int wd,                 /* image width */
  int ht,                 /* image height */
  char *name              /* file name */
)
{
  FILE   *fp;
  struct pxm_img img;
  long	i;

  fp = fopen(name,"rb");
  if (fp == NULL) {
    fprintf(stderr,"read_img: can't open file %s\n",name);
    exit (-1);
  }

  if( read_pxm_header (fp, &img) ) {
    fprintf (stderr, "read_img: error reading header for input file %s\n",name);
    exit (-1);
  }

  if(img.pxm_type != 'g') {
    fprintf(stderr,"read_img: input file %s must be pgm\n",name); 
    exit(1); 
  }
  if((img.width != wd)||(img.height!=ht)) {
    fprintf(stderr,"read_img: input file %s has inconsistant width or height\n",name); 
    exit(1); 
  }

  for(i=0; i<ht; i++) {
    if(wd!=fread((unsigned char *)image[i],sizeof(char),wd,fp)) {
      fprintf(stderr,"read_img: input file %s corrupted\n",name); 
      exit(1);
    }
  }

  fclose(fp);
}



