/*
* All questions regarding the software should be addressed to
*
*       Prof. Charles A. Bouman
*       Purdue University
*       School of Electrical and Computer Engineering
*       1285 Electrical Engineering Building
*       West Lafayette, IN 47907-1285
*       USA
*       +1 765 494 0340
*       +1 765 494 3358 (fax)
*       email:  bouman@ecn.purdue.edu
*       http://www.ece.purdue.edu/~bouman
*
* Copyright (c) 1995 The Board of Trustees of Purdue University.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement is
* hereby granted, provided that the above copyright notice and the following
* two paragraphs appear in all copies of this software.
*
* IN NO EVENT SHALL PURDUE UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF PURDUE UNIVERSITY HAS
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* PURDUE UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
* AND PURDUE UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
* UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/
#include "solve.h"

double  solve(
  double (*f)(), /* pointer to function to be solved */
  double a,      /* minimum value of solution */
  double b,      /* maximum value of solution */
  double err,    /* accuarcy of solution */
  int *code      /* error code */
  )
/* Solves equation (*f)(x) = 0 on x in [a,b]. Uses half interval method.*/
/* Requires that (*f)(a) and (*f)(b) have opposite signs.		*/
/* Returns code=0 if signs are opposite.				*/
/* Returns code=1 if signs are both positive. 				*/
/* Returns code=-1 if signs are both negative. 				*/
{
  int     signa,signb,signc;
  double  fa,fb,fc,c,signaling_nan();
  double  dist;

  fa = (*f)(a);  signa = fa>0;
  fb = (*f)(b);  signb = fb>0;

/* check starting conditions */
  if( signa==signb ) {
    if(signa==1) *code = 1;
    else *code = -1;
    return(0.0);
    }
  else *code = 0;

/* half interval search */
  if( (dist=b-a)<0 ) dist = -dist;
  while(dist>err) {
    c = (b+a)/2;
    fc = (*f)(c);  signc = fc>0;
    if(signa == signc) { a = c; fa = fc; }
    else { b = c; fb = fc; }
    if( (dist=b-a)<0 ) dist = -dist;
    }

/* linear interpolation */
  if( (fb-fa)==0 ) return(a);
  else {
    c = (a*fb - b*fa)/(fb-fa);
    return(c);
    }
}

